local MAJOR = "LibScriptableDisplayAVSSuperScope-1.0"
local MINOR = 14
assert(LibStub, MAJOR.." requires LibStub")
local LibAVSSuperScope = LibStub:NewLibrary(MAJOR, MINOR)
if not LibAVSSuperScope then return end
local PluginBlend = LibStub("LibScriptableDisplayPluginBlend-1.0", true)
assert(PluginBlend, MAJOR .. " requires LibScriptableDisplayBlend-1.0")
local LibError = LibStub("LibScriptableDisplayError-1.0")
assert(LibError, MAJOR .. " requires LibScriptableDisplayError-1.0")

local pool = setmetatable({}, {__mode = "k"})

if not LibAVSSuperScope.__index then
	LibAVSSuperScope.__index = LibAVSSuperScope
end

-- @name LibScriptableDisplayAVSSuperScope:New
-- @usage LibScriptableDisplayAVSSuperScope:New(name, size, defval, errorLevel)
-- @param name A name for the buffer
-- @param config This object's parameters
-- @param errorLevel The verbocity level
-- @return A new LibScriptableDisplayAVSSuperScope object

function LibAVSSuperScope:New(name, config, width, height, errorLevel)
	assert(type(name) ~= "nil", "LibAVSSuperScope requires a name")
	assert(type(config) == "table", "LibAVSSuperScope requires a config")

	local obj = next(pool)

	if obj then
		pool[obj] = nil
	else
		obj = {}
	end

	setmetatable(obj, self)

	obj.name = name

	obj.init = Evaluator.ExecuteCode(self, MAJOR .. " init", config.init, true, "", true)
	obj.beat = Evaluator.ExecuteCode(self, MAJOR .. " beat", config.beat, true, "", true)
	obj.frame = Evaluator.ExecuteCode(self, MAJOR .. " frame", config.frame, true, "", true)
	obj.point = Evaluator.ExecuteCode(self, MAJOR .. " point", config.point, true, "", true)

	obj.width = width
	obj.height = height

	obj.n = 100
	obj.b = 0
	obj.x = 0
	obj.y = 0
	obj.i = 0
	obj.v = 0
	obj.w = 0
	obj.h = 0
	obj.t = 0
	obj.d = 0
	obj.red = 0
	obj.green  = 0
	obj.blue = 0
	obj.linesize = 0
	obj.skip = 0
	obj.drawmode = 0

	obj.buffer = LibBuffer:New(MAJOR .. " buffer", obj.width * obj.height, 0, errorLevel)

	return obj

end

-- @name LibScriptableDisplayAVSSuperScope:Del
-- @usage LibScriptableDisplayAVSSuperScope:Del([lcd]) or object:Del()
-- @param lcd An optional lcd object
-- @return Nothing
function LibAVSSuperScope:Del(buff)
	if not buff then buff = self end
	pool[buff] = true
	buff.error:Del()
	for i = 0, buff:Size() do
		buff.buffer[i] = false
	end
	buff.size = 0
	del(buff.buffer)
end

function LibAVSSuperScope:Render()
	local isBeat
	local buf = LibBuffer:New("pcmbuf", 512, 0)

    if(self.needs_init) then
        self.needs_init = false;
		self.init()
    end


    local a, l
	local lx, ly, x, y = 0, 0, 0, 0
	local current_color = 0xffffff
	local ws = 1
	local xorv = (ws * 128) ^ 128
	local fa_data = {}

	for i = 0, 512 do
		buf.buffer[i] = random(100) / 100
	end

	local pcmbuf = buf:MovingAverageExp(0.5)

    self.h = self.height;
    self.w = self.width;
    self.b = isBeat
    self.blue = bit.band(current_color, 0xff)/255.0;
    self.green = bit.band(bit.rshift(current_color, 8), 0xff)/255.0;
    self.red = bit.band(bit.rshift(current_color, 16), 0xff)/255.0;
    self.skip = 0.0;
    self.linesize = 3
    self.drawmode = 1 --self.draw_type ? 1.0 : 0.0;

    --scope_run(self, SCOPE_RUNNABLE_FRAME);
	self.frame()

    if (isBeat) then
        --scope_run(self, SCOPE_RUNNABLE_BEAT);
		self.beat()
    end

    l = self.n;
    if (l > 128*1024) then
        l = 128*1024;
	end

    if( l == 0 or l == 1 ) then
        l = 2;
	end

    for a=0, l - 1 do
        --//int x, y;
        --//double r=(a*576.0)/l;
        --//double s1=r-(int)r;
        --//double yr=((int)pcmbuf[(int)r]^xorv)*(1.0f-s1)+((int)pcmbuf[(int)r+1]^xorv)*(s1);
        --//self->v = yr/128.0 - 1.0;
        self.v = pcmbuf[a * 288 / l];
        self.i = a/(l-1);
        self.skip = 0.0;

		self.point()

        x = (int)((self.x + 1.0) * width * 0.5);
        y = (int)((self.y + 1.0) * height * 0.5);

        if (self.skip < 0.00001) then

			local this_color = bit.bor(self.blue, bit.bor(bit.lshift(self.green, 8), bit.lshift(self.red, 16)))

			if true then --self->drawmode < 0.00001) then
				if (y >= 0 and y < self.height and x >= 0 and x < self.width) then
					local sub = self.buffer:Sub(y * self.width + x)
					PluginBlend.BLEND_LINE(self.line_blend_mode, sub, this_color);
					self.buffer.buffer[y * self.width + x] = sub.buffer[1]
					sub:Del()
				end
			else--[[
				if (a > 0) then
					if (y >= 0 && y < video->height && x >= 0 && x < video->width &&
						ly >= 0 && ly < video->height && lx >= 0 && lx < video->width) {
							VisColor color;
							visual_color_from_uint32(&color, this_color);
							avs_gfx_line_ints(video, lx, ly, x, y, &color);
					end
				end]]
			end
			lx = x;
			ly = y;
		end
    end

    return 0;
end

